<?php

namespace Drupal\cmlmigrations\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\cmlapi\Controller\GetCml;

/**
 * InitMigrations.
 */
class MigrationsController extends ControllerBase {

  /**
   * Init.
   */
  public static function init($cml = FALSE) {
    $result = TRUE;
    // Берём актуальный обмен.
    if (!$cml) {
      $cml = GetCml::actual();
    }
    $config = \Drupal::service('config.factory')->getEditable('cmlmigrations.settings');
    $config->set('runing_cml', $cml->id())->save();
    $status = $cml->getState();
    if (in_array($status, ['new', 'progress'])) {
      // Check previous exchanges $cml.
      $progress = self::query('progress');
      if ($status == 'new' && !empty($progress)) {
        $cml = $progress;
      }
      // Opts: progress, success, failure.
      if ($migrations = MigrationsHelper::getMigrations()) {
        $result = self::import($cml, $migrations);
      }
    }
    $config->set('runing_cml', '')->save();
    return $result;
  }

  /**
   * Import.
   */
  private static function import($cml, $migrations) {
    $id = $cml->id();
    $config = \Drupal::config('cmlmigrations.settings');
    // Таймаут default: 1 час.
    $timeout_min = $config->get('timeout');
    $timeout = $cml->changed->value + 60 * $timeout_min;
    $status = $cml->getState();
    \Drupal::logger(__CLASS__)->notice("go: $id = $status");
    // Первое обращение к обмену - меняем статус и запускаем.
    if ($status == 'new') {
      // Миграции готовы к запуску.
      if ($migrations['status']) {
        // Меняем статус.
        \Drupal::logger(__CLASS__)->notice("ok: $id = $status > progress (exec)");
        $cml->setState('progress');
        $cml->save();
        // Запускаем (Дата changed равна времени изменения).
        MigrationsExec::exec('nohup');
        sleep(5);
        return 'progress';
      }
      else {
        // Проверяем статус.
        if (self::queryQuickRun($id, $config->get('timeout-quick-run'))) {
          $msg = "fail: $id = $status > migrations busy";
          \Drupal::logger(__CLASS__)->warning($msg);
          return 'progress';
        }
        else {
          // Если мы попали сюда - где-то ошибка.
          $msg = "fail: $id = $status > migrations busy";
          \Drupal::logger(__CLASS__)->error($msg);
          $cml->setState('busy');
          $cml->save();
          return 'failure';
        }

      }
    }
    // Уже импортируется.
    elseif ($status == 'progress') {
      // Миграция закончилась.
      if ($migrations['status']) {
        \Drupal::logger(__CLASS__)->notice("ok: $id = $status > success (done)");
        $cml->setState('success');
        $cml->save();
        return 'success';
      }
      // Миграция ещё не закончилась.
      if ($timeout > REQUEST_TIME) {
        // Пока ок, работаем.
        $time = format_date($timeout - REQUEST_TIME, 'custom', 'H:i:s');
        \Drupal::logger(__CLASS__)->notice("progress: $id / Time: $time");
        return 'progress';
      }
      else {
        // Время вышло.
        \Drupal::logger(__CLASS__)->error("fail: $id = $status > timeout");
        $cml->setState('failure');
        $cml->save();
        return 'failure';
      }
    }
    else {
      // Если мы попали сюда - где-то ошибка.
      \Drupal::logger(__CLASS__)->error("fail: $id = $status WTF?");
      $cml->setState('failure');
      $cml->save();
    }
    return 'failure';
  }

  /**
   * Query Quick Runner Check TRUE/FALSE.
   */
  public static function queryQuickRun($id, $timeout) {
    if (!is_numeric($timeout)) {
      $timeout = 0;
    }
    $entities = [];
    $entity_type = 'cml';
    $storage = \Drupal::entityManager()->getStorage($entity_type);
    $query = \Drupal::entityQuery($entity_type)
      ->condition('status', 1)
      ->condition('id', $id, '!=')
      ->condition('changed', REQUEST_TIME - $timeout, '>')
      ->condition('type', 'catalog')
      ->condition('state', ['success'], 'IN')
      ->sort('created', 'ASC');
    $ids = $query->execute();
    if (empty($ids)) {
      return FALSE;
    }
    else {
      return TRUE;
    }
  }

  /**
   * Query.
   */
  public static function query($status) {
    $entities = [];
    $entity_type = 'cml';
    $storage = \Drupal::entityManager()->getStorage($entity_type);
    $query = \Drupal::entityQuery($entity_type)
      ->condition('status', 1)
      ->condition('state', [$status], 'IN')
      ->sort('created', 'ASC');
    $ids = $query->execute();
    if (!empty($ids)) {
      foreach ($storage->loadMultiple($ids) as $id => $entity) {
        $entities[$id] = $entity;
      }
    }
    return array_shift($entities);
  }

}
